from typing import Optional, Dict, List, Any
import httpx
from pydantic import BaseModel
from lume_py.models.endpoints.config import get_settings
from lume_py.models.endpoints.jobs import Job
from lume_py.models.endpoints.workshop import WorkShop
from lume_py.models.endpoints.mappers import Mapper
from lume_py.models.pipeline import PipelinePopulateSheets

settings = get_settings()


class Pipeline(BaseModel):
    id: Optional[str] = None
    user_id: Optional[str] = None
    name: Optional[str] = None
    description: Optional[str] = None
    target_schema_id: Optional[str] = None
    source_schema_id: Optional[str] = None
    updated_at: Optional[str] = None
    created_at: Optional[str] = None
    target_schema: Optional[Dict[str, Any]] = None

    class Config:
        orm_mode = True

    @staticmethod
    async def get_pipelines_data_page(page: int = 1, size: int = 50) -> List['Pipeline']:
        response = await settings.client.fetch_paginated_data('pipelines', page, size)
        return [Pipeline(**item) for item in response['items']]

    @classmethod
    async def create(cls, name: str, target_schema: Dict[str, Any], description: Optional[str] = None) -> 'Pipeline':
        payload = {
            "name": name,
            "target_schema": target_schema,
            "description": description
        }
        response = await settings.client.post('pipelines', payload)
        return cls(**response)

    @classmethod
    async def get_pipeline_by_id(cls, pipeline_id: str) -> 'Pipeline':
        response = await settings.client.get(f'pipelines/{pipeline_id}')
        return cls(**response)

    async def update(self, name: str, description: str) -> 'Pipeline':
        if not self.id:
            raise ValueError("Pipeline ID is required for updating.")
        payload = {'name': name, 'description': description}
        response = await settings.client.put(f'pipelines/{self.id}', payload)
        return Pipeline(**response)

    async def delete(self) -> None:
        if not self.id:
            raise ValueError("Pipeline ID is required for deletion.")
        await settings.client.delete(f'pipelines/{self.id}')

    async def create_job(self, source_data: List[Dict[str, Any]]) -> Job:
        if not self.id:
            raise ValueError("Pipeline ID is required for creating a job.")
        response = await settings.client.post(f'pipelines/{self.id}/jobs', {'data': source_data})
        return Job(**response)

    async def get_workshops(self, page: int = 1, size: int = 50) -> List[WorkShop]:
        if not self.id:
            raise ValueError("Pipeline ID is required for fetching workshops.")
        response = await settings.client.fetch_paginated_data(f'pipelines/{self.id}/workshops', page, size)
        return [WorkShop(**workshop) for workshop in response['items']]

    async def create_workshop(self) -> WorkShop:
        if not self.id:
            raise ValueError("Pipeline ID is required for creating a workshop.")
        response = await settings.client.post(f'pipelines/{self.id}/workshops')
        return WorkShop(**response)

    async def get_target_schema(self) -> Dict[str, Any]:
        if not self.id:
            raise ValueError("Pipeline ID is required for fetching target schema.")
        return await settings.client.get(f'pipelines/{self.id}/target_schema')

    async def get_mapper(self) -> Dict[str, Any]:
        if not self.id:
            raise ValueError("Pipeline ID is required for fetching mapper.")
        response = await settings.client.get(f'pipelines/{self.id}/mapper')
        if response is None:
            raise ValueError("No mapper found for this pipeline, consider running the job first.")
        return Mapper(**response)
    

    async def learn(self, target_property_names: Optional[List[str]] = None) -> None:
        if not self.id:
            raise ValueError("Pipeline ID is required for learning.")
        payload = {'target_field_names': target_property_names}
        await settings.client.post(f'pipelines/{self.id}/learn', payload)

    async def run_pipeline(self, source_data: List[Dict[str, Any]]) -> Mapper:
        if not self.id:
            raise ValueError("Pipeline ID is required for running the pipeline.")
        response = await settings.client.post(f'pipeline/{self.id}/run', {'data': source_data})
        status = response['status']
        result_id = response['id']
        while status in ['queued', 'running']:
            result = await settings.client.get(f'mappings/{result_id}')
            status = result['status']
        if result is None:
            raise ValueError("No mapper found for this pipeline, consider running the job first.")
        return Mapper(**result)

    async def upload_sheets(self, file_path: str, pipeline_map_list: Optional[str] = '', second_table_row_to_insert: Optional[int] = None):
        with open(file_path, 'rb') as file:
            files = {'file': (file_path, file, 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet')}
            data = {
                'pipeline_map_list': pipeline_map_list,
                'second_table_row_to_insert': second_table_row_to_insert
            }
            async with httpx.AsyncClient(timeout=60) as cli:
                response = await cli.post('https://api.lume.ai/crud/pipelines/upload/sheets', headers={'lume-api-key': settings.lume_api_key}, files=files, data=data)
                response.raise_for_status()
                return response.json()

    async def populate_sheets(self, pipeline_ids: str, populate_excel_payload: str, file_type: str) -> Dict[str, Any]:
        sheets_data = PipelinePopulateSheets(pipeline_ids=pipeline_ids, populate_excel_payload=populate_excel_payload, file_type=file_type)
        return await settings.client.post('pipelines/populate/sheets', sheets_data.model_dump())

    async def get_images(self) -> Dict[str, Any]:
        if not self.id:
            raise ValueError("Pipeline ID is required for fetching images.")
        return await settings.client.post(f'pipelines/{self.id}/populate/images')



from typing import List, Dict, Any, Optional
from pydantic import BaseModel
from lume_py.models.endpoints.config import get_settings

settings = get_settings()

class Target(BaseModel):
    id: Optional[str] = None
    user_id: Optional[str] = None
    name: Optional[str] = None
    filename: Optional[str] = None
    schema: Optional[Dict[str, Any]] = None

    class Config:
        orm_mode = True

    @staticmethod
    async def get(page: int = 1, size: int = 50) -> List['Target']:
        """
        Retrieves all target schemas.
        :return: List of target schemas.
        """
        response = await settings.client.fetch_paginated_data('target_schemas', page, size)
        return [Target(**item) for item in response['items']]

    @staticmethod
    async def create(target_schema: Dict[str, Any], name: str = "string",  filename: str = "string") -> 'Target':
        """
        Creates a new target schema.
        :param target_schema: The schema to create.
        :param name: The name of the schema.
        :param description: The description of the schema.
        :param filename: The filename of the schema.
        :return: The created target schema.
        """
        payload = {
            'name': name,
            'schema': target_schema,
            'filename': filename
        }
        response = await settings.client.post('target_schemas', payload)
        return Target(**response)

    @classmethod
    async def get_schema_by_id(cls, target_schema_id: str) -> Dict[str, Any]:
        """
        Retrieves a target schema by its ID.
        :param target_schema_id: The ID of the target schema to retrieve.
        :return: The target schema details.
        """
        response = await settings.client.get(f'target_schemas/{target_schema_id}')
        return response

    @staticmethod
    async def get_target_by_id(target_id, page: int = 1, size: int = 50) -> List['Target']:
        """
        Retrieves all target schemas.
        :return: List of target schemas.
        """
        response = await Target.get(page, size)
        for target in response:
            if target.id == target_id:
                return target
    
    async def get_schema(self) -> Dict[str, Any]:
        """
        Retrieves the details of this target schema.
        :return: The target schema details.
        """
        response = await settings.client.get(f'target_schemas/{self.id}')
        return response

    async def delete(self) -> None:
        """
        Deletes a specific target schema by its ID.
        """
        await settings.client.delete(f'target_schemas/{self.id}')

    async def update(self, name: str = "string", filename: str = "string", target_schema: Dict[str, Any] = {}) -> 'Target':
        """
        Updates an existing target schema with the provided details.
        :param name: The new name of the schema.
        :param description: The new description of the schema.
        :param filename: The new filename of the schema.
        :param target_schema: The new target schema.
        :return: The updated target schema details.
        """
        payload = {
            'name': name,
            'filename': filename,
            'schema': target_schema
        }
        response = await settings.client.put(f'target_schemas/{self.id}/update', data=payload)
        return response

    async def get_target_schema_object(self) -> 'Target':
        """
        Retrieves the object of a specific target schema by its ID.
        :return: The target schema object.
        """
        response = await settings.client.get(f'target_schemas/{self.id}/object')
        return Target(**response)

    @staticmethod
    async def generate_target_schema(sample: Dict[str, Any]) -> Dict[str, Any]:
        """
        Generates a new target schema.
        :param sample: The sample data to generate the schema.
        :return: The generated target schema details.
        """
        response = await settings.client.post('target_schemas/generate', {'sample': sample})
        return response



import sys
import os
import pytest
import json
import httpx

# Add the parent directory to the sys.path
sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), '..', '..')))

import lume_py

# Load target and source data from files
target_data_path = os.path.join(os.path.dirname(__file__), 'target.json')
with open(target_data_path, encoding='utf-8') as f:
    target_data = json.load(f)

source_data_path = os.path.join(os.path.dirname(__file__), 'source.json')
with open(source_data_path, encoding='utf-8') as f:
    source_data = json.load(f)


@pytest.mark.asyncio
async def test_pipeline_workflow():
    lume_py.set_api_key('222be6ab26bf6ea6bb86fffbb3880a67')
    
    # Print the payload to debug the structure
    print("Payload being sent for pipeline creation:", json.dumps(target_data, indent=2))
    
    # Attempt to create the pipeline
    try:
        pipeline = await lume_py.Pipeline.create(name="testingsdktwelve", description="test", target_schema=target_data)
        assert pipeline is not None
        assert pipeline.name == "testingsdktwelve"

        pipeline_id = pipeline.id
        fetched_pipeline = await lume_py.Pipeline.get_pipeline_by_id(pipeline_id)
        assert fetched_pipeline.id == pipeline_id

        updated_pipeline = await pipeline.update(name="updated_pipeline_test_twelve", description="A test pipeline updated")
        assert updated_pipeline.name == "updated_pipeline_test_twelve"

        paginated_pipelines = await pipeline.get_pipelines_data_page()
        assert paginated_pipelines is not None

        job = await lume_py.Job.create(pipeline_id=pipeline_id, source_data=source_data)
        assert job is not None

        job_result = await job.run()
        assert job_result is not None

    except httpx.HTTPStatusError as exc:
        print(f"Error response {exc.response.status_code} while requesting {exc.request.url!r}.")
        print(exc.response.json())
        raise

@pytest.mark.asyncio
async def test_pdf_workflow():
    lume_py.set_api_key('222be6ab26bf6ea6bb86fffbb3880a67')
    
    pdf_path = '/Users/aryanpanda/Downloads/PDF/Alaris/ADV/226583 (1).pdf'
    
    try:
        pdf_service = lume_py.PDF()
        extracted_pdf = await pdf_service.extract_pdf(pdf_path=pdf_path)
        assert extracted_pdf is not None

        adv_url = await pdf_service.get_adv_url(pdf_id='ebb5d33c-e7b9-464c-bc47-3776fa9c6c22')
        assert adv_url.startswith('https://')

        pdf_url = await pdf_service.get_pdf_url(pdf_id='509ae017-e566-4d1b-a451-87d96fe65f4c')
        assert pdf_url.startswith('https://')

    except httpx.HTTPStatusError as exc:
        print(f"Error response {exc.response.status_code} while requesting {exc.request.url!r}.")
        print(exc.response.json())
        raise

@pytest.mark.asyncio
async def test_result_workflow():
    lume_py.set_api_key('222be6ab26bf6ea6bb86fffbb3880a67')
    
    try:
        result = await lume_py.Result.get_results()
        assert result is not None

        spec = await result[0].get_spec()

        assert spec is not None

    except httpx.HTTPStatusError as exc:
        print(f"Error response {exc.response.status_code} while requesting {exc.request.url!r}.")
        print(exc.response.json())
        raise

@pytest.mark.asyncio
async def test_target_workflow():
    lume_py.set_api_key('222be6ab26bf6ea6bb86fffbb3880a67')
    
    try:
        target = await lume_py.Target.create(name='testing_target_sdk_4', filename='testing target sdk 3', target_schema=target_data)
        assert target is not None

        target_id = target.id
        fetched_target = await lume_py.Target.get_target_by_id(target_id)
        assert fetched_target is not None

        updated_target = await target.update(target_schema=target_data, name="updatedNew")
        
        assert updated_target is not None

    except httpx.HTTPStatusError as exc:
        print(f"Error response {exc.response.status_code} while requesting {exc.request.url!r}.")
        print(exc.response.json())
        raise

@pytest.mark.asyncio
async def test_workshop_workflow():
    lume_py.set_api_key('222be6ab26bf6ea6bb86fffbb3880a67')
    
    try:
        pipeline = await lume_py.Pipeline.get_pipeline_by_id('63e7ff0e-031d-48b6-9f58-b162e03f63a1')
        workshop = await pipeline.create_workshop()
        assert workshop is not None

    except httpx.HTTPStatusError as exc:
        print(f"Error response {exc.response.status_code} while requesting {exc.request.url!r}.")
        print(exc.response.json())
        raise


if __name__ == "__main__":
    pytest.main(["-v", __file__])



from typing import Dict, List, Any, Optional
from pydantic import BaseModel
from lume_py.models.endpoints.config import get_settings
from lume_py.models.endpoints.workshop import WorkShop
from lume_py.models.endpoints.results import Result

settings = get_settings()

class Job(BaseModel):
    id: Optional[str] = None
    user_id: Optional[str] = None
    pipeline_id: Optional[str] = None
    target_schema_id: Optional[str] = None
    source_schema_id: Optional[str] = None
    status: Optional[str] = None
    created_at: Optional[str] = None
    updated_at: Optional[str] = None

    class Config:
        orm_mode = True

    @staticmethod
    async def get_jobs_data_page(page: int = 1, size: int = 50) -> List['Job']:
        response = await settings.client.fetch_paginated_data('jobs', page, size)
        return [Job(**item) for item in response['items']]

    @classmethod
    async def create(cls, pipeline_id: str, source_data: List[Dict[str, Any]]) -> 'Job':
        response = await settings.client.post(f'pipelines/{pipeline_id}/jobs', {'data': source_data})
        return cls(**response)

    @classmethod
    async def get_job_by_id(cls, job_id: str) -> 'Job':
        response = await settings.client.get(f'jobs/{job_id}')
        return cls(**response)

    async def delete(self) -> None:
        if not self.id:
            raise ValueError("Job ID is required for deletion.")
        await settings.client.delete(f'jobs/{self.id}')

    async def run(self) -> Result:
        response = await settings.client.post(f'jobs/{self.id}/run')
        status = response['status']
        result_id = response['id']

        while status in ['queued', 'running']:
            result = await settings.client.get(f'results/{result_id}')
            status = result['status']
        return Result(**result)

    async def get_workshops(self, page: int = 1, size: int = 50) -> List[WorkShop]:
        response = await settings.client.fetch_paginated_data(f'jobs/{self.id}/workshops', page, size)
        return [WorkShop(**workshop) for workshop in response['items']]

    async def get_target_schema(self) -> Dict[str, Any]:
        return await settings.client.get(f'jobs/{self.id}/target_schema')

    @classmethod
    async def create_and_run(cls, pipeline_id: str, source_data: List[Dict[str, Any]]) -> Result:
        job = await cls.create(pipeline_id, source_data)
        return await job.run()

    async def get_results(self, page: int = 1, size: int = 50) -> List[Result]:
        response = await settings.client.fetch_paginated_data(f'jobs/{self.id}/results', page, size)
        return [Result(**result) for result in response['items']]



from typing import List, Dict, Any, Optional
from lume_py.models.endpoints.config import get_settings
from lume_py.models.endpoints.results import Result

from pydantic import BaseModel, TypeAdapter

resultAdaptar = TypeAdapter(Result)
settings = get_settings()

class WorkShop(BaseModel):
    id: Optional[str] = None
    user_id: Optional[str] = None
    pipeline_id: Optional[str] = None
    job_id: Optional[str] = None
    target_schema_id: Optional[str] = None
    source_schema_id: Optional[str] = None
    status: Optional[str] = None
    created_at: Optional[str] = None
    updated_at: Optional[str] = None

    class Config:
        orm_mode = True

    @staticmethod
    async def get_workshops(page: int = 1, size: int = 50) -> List['WorkShop']:
        """
        Fetches all workshop data.
        :param page: The page number to fetch (optional, defaults to 1).
        :param size: The number of items per page (optional, defaults to 50).
        :return: A paginated response of workshops.
        """
        response = await settings.client.fetch_paginated_data('workshops', page, size)
        return [WorkShop(**item) for item in response['items']]

    @classmethod
    async def get_by_id(cls, workshop_id: str) -> 'WorkShop':
        """
        Retrieves details of a specific workshop.
        :param workshop_id: The ID of the workshop to fetch details for.
        :return: Workshop details.
        """
        response = await settings.client.get(f'workshops/{workshop_id}')
        return cls(**response)

    async def get_details(self) -> 'WorkShop':
        """
        Retrieves the details of this workshop.
        :return: The workshop details.
        """
        response = await settings.client.get(f'workshops/{self.id}')
        return WorkShop(**response)

    async def delete(self) -> Dict[str, Any]:
        """
        Deletes a workshop with the specified ID.
        :return: Success message on successful deletion.
        """
        return await settings.client.delete(f'workshops/{self.id}')

    async def run_mapper(self, mapper: List[Dict[str, Any]]) -> Dict[str, Any]:
        """
        Runs the mapper of a workshop with the specified ID.
        :param mapper: Details required for running the mapper.
        :return: The result of running the mapper.
        """
        response = await settings.client.post(f'workshops/{self.id}/mapper/run', data={'mapper': mapper})
        return response

    async def run_sample(self, sample: Dict[str, Any]) -> Dict[str, Any]:
        """
        Runs a sample for the workshop with the specified ID.
        :param sample: Details required for running the sample.
        :return: The result of running the sample.
        """
        response = await settings.client.post(f'workshops/{self.id}/sample/run', data={'sample': sample})
        response_status = response['status']
        while response_status in ['queued', 'running']:
            response = await settings.client.get(f'results/{response["id"]}')
            response_status = response['status']
        return response

    async def run_target_schema(self, target_schema: Dict[str, Any]) -> Dict[str, Any]:
        """
        Runs the target schema for the workshop with the specified ID.
        :param target_schema: Details required for running the target schema.
        :return: The result of running the target schema.
        """
        # implement polling 
        response = await settings.client.post(f'workshops/{self.id}/target_schema/run', data={'target_schema': target_schema})
        response_status = response['status']
        while response_status in ['queued', 'running']:
            response = await settings.client.get(f'results/{response["id"]}')
            response_status = response['status']
        
        return response

    async def run_prompt(self, target_fields_to_prompt: Dict[str, Any]) -> Dict[str, Any]:
        """
        Runs the prompts for the workshop with the specified ID.
        :param target_fields_to_prompt: Details required for running the prompt.
        :return: The result of running the prompt.
        """
        response = await settings.client.post(f'workshops/{self.id}/prompt/run', data={'target_fields_to_prompt': target_fields_to_prompt})
        response_status = response['status']
        while response_status in ['queued', 'running']:
            response = await settings.client.get(f'results/{response["id"]}')
            response_status = response['status']

        return response

    async def deploy(self) -> Dict[str, Any]:
        """
        Deploys the workshop with the specified ID.
        :return: The deployed workshop details.
        """
        response = await settings.client.post(f'workshops/{self.id}/deploy')
        return WorkShop(**response)

    async def get_results(self, page: int = 1, size: int = 50) -> List[Result]:
        """
        Retrieves results associated with a specific workshop.
        :param page: The page number to fetch (optional, defaults to 1).
        :param size: The number of items per page (optional, defaults to 50).
        :return: A paginated response of results.
        """
        response = await settings.client.fetch_paginated_data(f'workshops/{self.id}/results', page, size)
        return [Result(**item) for item in response['items']]

    async def get_target_schema(self) -> Dict[str, Any]:
        """
        Retrieves the target schema for a specific workshop.
        :return: The target schema for the workshop.
        """
        return await settings.client.get(f'workshops/{self.id}/target_schema')



from pydantic import BaseModel, Field
from typing import Optional, Dict, Any, Tuple

class Pipelines(BaseModel):
    id: Optional[str] = None
    user_id: Optional[str] = None
    name: Optional[str] = None
    description: Optional[str] = None
    target_schema_id: Optional[str] = None
    source_schema_id: Optional[str] = None
    updated_at: Optional[str] = None
    created_at: Optional[str] = None

    class Config:
        orm_mode = True

class PipelineCreatePayload(BaseModel):
    name: str = Field(..., example="Example Pipeline")
    description: str = Field(None, example="This is an example pipeline.")
    target_schema: Dict[str, Any] = Field(..., example={
        "type": "object",
        "properties": {
            "field": {"type": "string"}
        }
    })

    class Config:
        schema_extra = {
            "example": {
                "name": "Example Pipeline",
                "description": "This is an example pipeline.",
                "target_schema": {
                    "type": "object",
                    "properties": {
                        "field": {"type": "string"}
                    }
                }
            }
        }

        
        
class PipelineUpdatePayload(BaseModel):
    name: str
    description: str
    
    

class PipelineUploadSheets(BaseModel):
    file: Tuple[str, bytes, str]  
    pipeline_map_list: Optional[str] = Field(default=None, exclude=True) 
    second_table_row_to_insert: Optional[int] = Field(default=None, exclude=True)  

    class Config:
        schema_extra = {
            "example": {
                "file": ("example.xlsx", b"binarydata", "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet"),
                "pipeline_map_list": "map1,map2,map3",
                "second_table_row_to_insert": 5
            }
        }

class PipelinePopulateSheets(BaseModel):
    pipeline_ids: str
    populate_excel_payload: str
    file_type: str

    class Config:
        orm_mode = True
        


from typing import Dict, Optional
from pydantic import BaseModel

class Results(BaseModel):
    id: Optional[str] = None
    status: Optional[str] = None
    created_at: Optional[str] = None
    updated_at: Optional[str] = None
    

    class Config:
        orm_mode = True

class ResultMapper(BaseModel):
    result_id: Optional[str] = None
    index: Optional[int] = None
    source_record: Optional[Dict] = None
    mapped_record: Optional[Dict] = None
    messsage: Optional[str] = None

    class Config:
        orm_mode = True
        


from typing import Any, Optional, List, Dict
from pydantic import BaseModel
from lume_py.models.endpoints.config import get_settings
from lume_py.models.result import ResultMapper
from pydantic import TypeAdapter
import asyncio


settings = get_settings()

resultMapper = TypeAdapter(ResultMapper)

class Result(BaseModel):
    id: Optional[str] = None
    status: Optional[str] = None
    created_at: Optional[str] = None
    updated_at: Optional[str] = None

    class Config:
        orm_mode = True

    @staticmethod
    async def get_results(page: int = 1, size: int = 50) -> List['Result']:
        """
        Fetches all result data.
        """
        response = await settings.client.fetch_paginated_data('results', page, size)
        return [Result(**item) for item in response['items']]

    @classmethod
    async def get_by_id(cls, result_id: str) -> 'Result':
        """
        Retrieves a result by its ID.
        :param result_id: The ID of the result to retrieve.
        :return: The result details.
        """
        response = await settings.client.get(f'results/{result_id}')
        return cls(**response)

    async def get_details(self) -> 'Result':
        """
        Retrieves the details of this result.
        :return: The result details.
        """
        response = await settings.client.get(f'results/{self.id}')
        return Result(**response)

    async def get_spec(self) -> Dict[str, Any]:
        """
        Retrieves specifications associated with a specific result.
        :return: The specifications.
        """
        spec = await settings.client.get(f'results/{self.id}/spec')
        if spec:
            return spec
        else:
            raise ValueError("No spec found for this result, consider running the job first.")

    async def get_mappings(self) -> List[ResultMapper]:
        """
        Retrieves mappings associated with a specific result.
        :return: The list of mappings.
        """
        response = await settings.client.fetch_paginated_data(f'results/{self.id}/mappings')
        return [ResultMapper(**item) for item in response['items']]

    async def generate_confidence_scores(self, timeout: int = 10):
        """
        Generates confidence scores for a specific result.
        :param timeout: The timeout for the operation.
        :return: The confidence scores.
        """

        async def fetch_confidence_scores():
            confidence = await settings.client.post(f'results/{self.id}/confidence')
            status = confidence['status']
            while status in ['pending', 'running', 'queued']:
                confidence = await settings.client.get(f'results/{self.id}/confidence')
                status = confidence['status']
            return confidence

        try:
            confidence = await asyncio.wait_for(fetch_confidence_scores(), timeout)
        except Exception as exc:
            raise TimeoutError(f"Operation timed out after {timeout} seconds") from exc

        return confidence



import httpx
import pydantic
import pydantic_core
import pydantic_settings
from lume_py.models.endpoints.pipeline import Pipeline
from lume_py.models.endpoints.jobs import Job
from lume_py.models.endpoints.results import Result
from lume_py.models.endpoints.target import Target
from lume_py.models.endpoints.workshop import WorkShop
from lume_py.models.endpoints.mappers import Mapper
from lume_py.models.endpoints.config import Settings, get_settings
from lume_py.models.endpoints.excel import Excel
from lume_py.models.endpoints.pdf import PDF


settings = get_settings()

def set_api_key(api_key: str):
    settings.set_api_key(api_key)

__all__ = ['Pipeline', 'Job', 'Result', 'Target', 'WorkShop', 'Mapper', 'Settings', 'set_api_key', 'Excel', 'PDF', 'httpx', 'pydantic', 'pydantic_core', 'pydantic_settings', 'pydantic_core']



import httpx
from typing import Any, Dict, Optional

class Client:
    def __init__(self, api_key: str, base_url: str = 'https://staging.lume-terminus.com/crud/'):
        self.api_key = api_key
        self.base_url = base_url
        self.client = httpx.AsyncClient(
            base_url=self.base_url,
            headers={
                'Content-Type': 'application/json',
                'lume-api-key': self.api_key
            }
        )

    async def fetch_paginated_data(self, endpoint: str, page: int = 1, size: int = 50) -> Dict[str, Any]:
        response = await self.client.get(endpoint, params={'page': page, 'size': size})
        response.raise_for_status()
        return response.json()

    async def fetch_paginated_data_with_params(self, endpoint: str, params: Dict[str, Any]) -> Dict[str, Any]:
        response = await self.client.get(endpoint, params=params)
        response.raise_for_status()
        return response.json()

    async def post_paginated_data_with_params(self, endpoint: str, body: Dict[str, Any], page: int = 1, size: int = 50) -> Dict[str, Any]:
        response = await self.client.post(endpoint, json=body, params={'page': page, 'size': size})
        response.raise_for_status()
        return response.json()

    async def get(self, url: str, params: Optional[Dict[str, Any]] = None) -> Dict[str, Any]:
        response = await self.client.get(url, params=params)
        response.raise_for_status()
        return response.json()
    
    async def post(self, url: str, data: Optional[Dict[str, Any]] = None) -> Dict[str, Any]:
        response = await self.client.post(url, json=data)
        response.raise_for_status()
        return response.json()

    async def put(self, url: str, data: Optional[Dict[str, Any]] = None) -> Dict[str, Any]:
        response = await self.client.put(url, json=data)
        response.raise_for_status()
        return response.json()

    async def delete(self, url: str) -> Dict[str, Any]:
        response = await self.client.delete(url)
        response.raise_for_status()
        return response.json()
